assert( Necronomicon, "Necronomicon not found!")

----------------------------------
--        Are you local?        --
----------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Necronomicon")

local soulshardid = 6265
local soulshardcount = 0
local soulshardicon = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02"

local soulbagid = {
	[21340] = true, -- soul pouch
	[22243] = true, -- small soul pouch
	[22244] = true, -- box of souls
	[21342] = true, -- core felcloth bag
	[21872] = true, -- ebon shadowbag	
}

----------------------------------
--      Module Declaration      --
----------------------------------

NecronomiconSoulShard = Necronomicon:NewModule("SoulShard")
NecronomiconSoulShard.name = "SoulShard"

NecronomiconSoulShard.revision = tonumber(string.sub("$Revision: 27266 $", 12, -3))

NecronomiconSoulShard.defaults = {
	low = 5,
	destroy = false,
	posx = 200,
	posy = 200,
}

NecronomiconSoulShard.consoleOptions = {
	type = "group",
	name = L["Soul Shards"],
	desc = L["Options for Soul Shards."],
	args = {
		[L["Low"]] = {
			name = L["Low"], type = "range",
			desc = L["Set low on shards warning threshold."],
			get = function() return NecronomiconSoulShard.db.profile.low end,
			set = function(v)
				NecronomiconSoulShard.db.profile.low = v
			end,
			min = 0, max = 28, step = 1,
			disabled = function() return not NecronomiconSoulShard.core:IsModuleActive(NecronomiconSoulShard) end,
		},
		[L["Destroy"]] = {
			name = L["Destroy"], type = "toggle",
			desc = L["Set destroying of shards outside shard bags."],
			get = function() return NecronomiconSoulShard.db.profile.destroy end,
			set = function(v)
				NecronomiconSoulShard.db.profile.destroy = v
			end,
			disabled = function() return not NecronomiconSoulShard.core:IsModuleActive(NecronomiconSoulShard) end,
		},
	}
}

----------------------------------
--     Module Intialization     --
----------------------------------

function NecronomiconSoulShard:OnRegister()
	self:RegisterSpell("Drain Soul")
end

function NecronomiconSoulShard:OnEnable()
	soulshardcount = 0
	self:SetupButton()

	self:RegisterEvent("Necronomicon_SpellsUpdated")
	self:Necronomicon_SpellsUpdated()
	
	-- register the normal BAG_UPDATE for soulshards, it's not like the stones
	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "CountShards")
	self:CountShards()
end

----------------------------------
--        Event Handlers        --
----------------------------------

function NecronomiconSoulShard:CountShards()
	local oldshards = soulshardcount
	soulshardcount = GetItemCount( soulshardid )
	if soulshardcount ~= oldshards then
		self:DestroyShards()
		self.button.text:SetTextColor(0, 1, 0, 1)
		if soulshardcount < self.db.profile.low then
			self:Warn( string.format( L["Low on Shards! %d left!"], soulshardcount ) )
			self.button.text:SetTextColor(1, 0, 0, 1)
		end
		self.button.text:SetText( soulshardcount )
	end
end

function NecronomiconSoulShard:Necronomicon_SpellsUpdated()
	local spell = self:HasSpell("Drain Soul")
	if spell then
		self.button:SetAttribute("type1", "spell")
		self.button:SetAttribute("spell1", ( GetSpellName( spell.id, BOOKTYPE_SPELL) ) )
		self.button:Show()
	else
		self.button:SetAttribute("type1", "")
		self.button:SetAttribute("spell1", "")
		self.button:Hide()
	end
end

----------------------------------
--       Utility Functions      --
----------------------------------

function NecronomiconSoulShard:DestroyShards()
	-- destroy shards from normal bags

	if not self.db.profile.destroy then return end

	local bagid, bagitem, itemid

	for bag = 0, 4, 1 do
		bagid = ContainerIDToInventoryID(bag)
		_,_,bagitemid = string.find(GetInventoryItemLink("player", bagid) or "", "item:(%d+):%d+:%d+:%d+")
		if not soulbagid[bagitemid] then
			for slot = 1, GetContainerNumSlots(bag), 1 do
				_,_,itemid = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d+")
				itemid = tonumber(itemid)
				if itemid and itemid == soulshardid then
					PickupContainerItem(bag, slot)
					if( CursorHasItem()) then
						DeleteCursorItem()
					end
				end
			end
		end
	end

	soulshardcount = GetItemCount( soulshardid )
end

----------------------------------
--         GUI Functions        --
----------------------------------

function NecronomiconSoulShard:SetupButton()
	self:CreateButton()

	self.button.background:SetTexture( soulshardicon )

	self.button.text = self.button:CreateFontString(nil, "OVERLAY")
	self.button.text:SetFontObject(GameFontHighlightLarge)
	self.button.text:ClearAllPoints()
	self.button.text:SetTextColor(0, 1, 0, 1) 
	self.button.text:SetWidth(32)
	self.button.text:SetHeight(32)
	self.button.text:SetPoint("TOPLEFT", self.button, "TOPLEFT")
	self.button.text:SetJustifyH("CENTER")
	self.button.text:SetJustifyV("MIDDLE")
	self.button.text:SetText(0)

end
